"""ipdata.co service integration."""
import requests
from django_ip_geolocation.backends import GeolocationBackend
from django_ip_geolocation import settings


class IPDataCo(GeolocationBackend):
    """ipdata.co backend implementation."""

    def geolocate(self):
        """Call ipdata api."""
        api_key = settings.IP_GEOLOCATION_SETTINGS.get('BACKEND_API_KEY')

        if not api_key:
            msg = "BACKEND_API_KEY is required. Please provide an API_KEY in IP_GEOLOCATION_SETTINGS"  # noqa: E501
            raise Exception(msg)

        payload = {'api-key': api_key}
        url = 'https://api.ipdata.co/{}'.format(self._ip)
        res = requests.get(url, params=payload)
        if res.ok:
            self._raw_data = res.json()

    def _parse(self):
        """Parse raw data."""
        self._continent = self._raw_data.get('continent_name')
        self._country = {
            'code': self._raw_data.get('country_code'),
            'name': self._raw_data.get('country_name'),
        }

        self._geo_data = {
            'latitude': self._raw_data.get('latitude'),
            'longitude': self._raw_data.get('longitude'),
        }



"""Django view decorator."""

import logging
from django_ip_geolocation.utils import get_geolocation, set_cookie, \
    clean_geolocation_data
from django_ip_geolocation.settings import IP_GEOLOCATION_SETTINGS as _settings


def with_ip_geolocation(view_func):
    """Decorate a django view, to add geolocation data to request/response.

    :param: view_func: Django view function
    :type: function
    :return: A wrapper function
    :rtype: function
    """
    def inner(request):  # noqa: E501
        try:
            enable_request = _settings.get('ENABLE_REQUEST_HOOK')
            enable_response = _settings.get('ENABLE_RESPONSE_HOOK')
            enable_cookie = _settings.get('ENABLE_COOKIE', False)

            if not enable_request and not (enable_response or enable_cookie):
                return view_func(request)

            geolocation = get_geolocation(request)

            if enable_request:
                request.geolocation = geolocation

            response = view_func(request)

            if enable_response:
                header = _settings.get('RESPONSE_HEADER')
                response[header] = geolocation

            if enable_cookie:
                cleaned_geolocation_data = clean_geolocation_data(
                    geolocation, ['raw_data'])
                set_cookie(response, cleaned_geolocation_data)

            return response
        except Exception:
            logging.error('Django Ip Geolocation Error', exc_info=True)
            return view_func(request)

    return inner



"""Default settings for the module."""
from django.conf import settings  # noqa: E501 # pylint: disable=import-error

IP_GEOLOCATION_SETTINGS = {
    'BACKEND': 'django_ip_geolocation.backends.IPGeolocationAPI',
    'BACKEND_API_KEY': '',
    'BACKEND_EXTRA_PARAMS': {},
    'BACKEND_USERNAME': '',
    'RESPONSE_HEADER': 'X-IP-Geolocation',
    'ENABLE_REQUEST_HOOK': True,
    'ENABLE_RESPONSE_HOOK': True,
    'ENABLE_COOKIE': False,
    'FORCE_IP_ADDR': None,
}

CUSTOM_IP_GEOLOCATION_SETTINGS = getattr(settings, 'IP_GEOLOCATION_SETTINGS', {})  # noqa: E501

IP_GEOLOCATION_SETTINGS.update(CUSTOM_IP_GEOLOCATION_SETTINGS)



"""IP2location.com service integration."""

import requests
from django_ip_geolocation.backends import GeolocationBackend
from django_ip_geolocation.settings import IP_GEOLOCATION_SETTINGS as _settings


class IP2LocationCom(GeolocationBackend):
    """IP2location.com backend implementation."""

    def geolocate(self):
        """Call ip2location.com api."""
        api_key = _settings.get('BACKEND_API_KEY')
        extra_params = _settings.get('BACKEND_EXTRA_PARAMS', {})

        if not api_key:
            msg = "BACKEND_API_KEY is required. Please provide an API_KEY in IP_GEOLOCATION_SETTINGS"  # noqa: E501
            raise Exception(msg)

        payload = {
            'key': api_key,
            'ip': self._ip,
            'package': extra_params.get('package', 'WS24')
        }

        url = 'https://api.ip2location.com/v2/'
        res = requests.get(url, params=payload)
        if res.ok:
            self._raw_data = res.json()
        else:
            raise Exception(res.text)

    def _parse(self):
        """Parse raw data."""
        self._continent = self._raw_data.get('continent_name')
        self._country = {
            'code': self._raw_data.get('country_code'),
            'name': self._raw_data.get('country_name'),
        }

        self._geo_data = {
            'latitude': self._raw_data.get('latitude'),
            'longitude': self._raw_data.get('longitude'),
        }



"""All backends modules."""
from django_ip_geolocation.backends.base import GeolocationBackend
from django_ip_geolocation.backends.ipgeolocationapi import IPGeolocationAPI
from django_ip_geolocation.backends.ipstack import IPStack
from django_ip_geolocation.backends.ip2locationcom import IP2LocationCom
from django_ip_geolocation.backends.ipdataco import IPDataCo


__all__ = ['GeolocationBackend', 'IPGeolocationAPI',
           'IPStack', 'IP2LocationCom', 'IPDataCo']



"""Django middleware."""
import logging

from django.utils.deprecation import MiddlewareMixin  # noqa: E501 pylint: disable=import-error
from django_ip_geolocation.utils import get_geolocation, set_cookie, \
    clean_geolocation_data
from django_ip_geolocation.settings import IP_GEOLOCATION_SETTINGS as _settings  # noqa: E501


class IpGeolocationMiddleware(MiddlewareMixin):
    """Mixin Middleware Hook."""

    def __init__(self, get_response=None):  # noqa: D107
        self._geolocation_data = None
        super(IpGeolocationMiddleware, self).__init__(get_response)

    def process_request(self, request):
        """Process the request."""
        try:
            if not _settings.get('ENABLE_REQUEST_HOOK'):
                return

            self._get_geolocation(request)
            request.geolocation = self._geolocation_data
        except Exception:
            logging.error("Couldn't geolocate ip", exc_info=True)

    def process_response(self, request, response):
        """Process the response."""
        try:
            if not _settings.get('ENABLE_RESPONSE_HOOK') and \
                    not _settings.get('ENABLE_COOKIE'):
                return response

            if self._geolocation_data is None:
                self._get_geolocation(request)

            if _settings.get('ENABLE_REQUEST_HOOK'):
                # Response hook is enabled
                header = _settings.get('RESPONSE_HEADER')
                response[header] = self._geolocation_data

            if _settings.get('ENABLE_COOKIE'):
                cleaned_geolocation_data = clean_geolocation_data(
                    self._geolocation_data, ['raw_data'])
                set_cookie(response, cleaned_geolocation_data)

        except Exception:
            logging.error("Couldn't geolocate ip", exc_info=True)

        return response

    def _get_geolocation(self, request):
        """Fetch geolcation using backend defined in settings."""
        self._geolocation_data = get_geolocation(request)



"""Helper functions."""
import base64
import json
import logging
from django.utils.module_loading import import_string  # noqa: E501 # pylint: disable=import-error
from django_ip_geolocation import settings


def _get_remote_ip_from_request(request):
    """Retrieve remote ip addr from request.

    :param request: Django request
    :type: django.http.HttpRequest
    :return: Ip address
    :rtype: str
    """
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip_addr = x_forwarded_for.split(',')[0]
    else:
        ip_addr = request.META.get('REMOTE_ADDR')
    return ip_addr


def _get_geolocation_backend_cls():
    """Retun geolocation backend class.

    :return: Geolocation backend class
    :rtype: class
    """
    backend_class_name = settings.IP_GEOLOCATION_SETTINGS.get('BACKEND')
    geolocation_backend_cls = import_string(backend_class_name)
    return geolocation_backend_cls


def get_geolocation(request):
    """Fetch geolocation data.

    :param request: Django hooked request
    :type: django.http.HttpRequest
    :return: Geolocation data
    :rtype: dict
    """
    ip_addr = settings.IP_GEOLOCATION_SETTINGS.get('FORCE_IP_ADDR', None)
    if not ip_addr:
        ip_addr = _get_remote_ip_from_request(request)

    backend_cls = _get_geolocation_backend_cls()
    backend_instance = backend_cls(ip_addr)

    backend_instance.geolocate()
    return backend_instance.data()


def set_cookie(response, geolocation_data):
    """Add geolocation data in response cookie.

    :param response:
    :type: django.http.HttpResponse
    :param geolocation_data: Geolocation data
    :type: dict
    :rtype: None
    """
    encoded_data = base64.b64encode(json.dumps(geolocation_data))
    response.set_cookie('geolocation', encoded_data)


def clean_geolocation_data(geolocation_data, attr_to_remove=None):
    """Remove attributes from geolocation data.

    If no attributes are provided, return a copy of the same data.

    :param geolocation_data: Full geolocation data
    :type: dict
    :param attr_to_remove: List of attributes to remove
    :type: list
    :return: Geolocation data (cleaned or copy)
    :rtype: dict
    """
    geolocation_copy = geolocation_data.copy()

    if attr_to_remove is None:
        return geolocation_copy

    for attr in attr_to_remove:
        try:
            del geolocation_copy[attr]
        except KeyError:
            logging.info('Key not found, continuing ...')

    return geolocation_copy



"""Package setup script."""

from setuptools import setup, find_packages


def readme():
    """Load readme."""
    with open("README.md") as f:
        return f.read()


setup(
    name='django-ip-geolocation',
    version='1.5.1',
    author='Skander Ben Mahmoud',
    author_email='skander.bmahmoud@gmail.com',
    packages=find_packages(exclude=("tests", "docs")),
    url='https://github.com/rednaks/django-ip-geolocation',
    license='MIT',
    description="Django request/response hook (Middleware and Decorator) to geolocate visitors using their IP address",  # noqa: E501
    long_description_content_type='text/markdown',
    long_description=readme(),
    install_requires=[
        'requests>=1.0.4',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)



"""Base interface for backends."""


class GeolocationBackend(object):
    """Base interface for backends."""

    def __init__(self, ip):
        """Construct class.

        :param ip: Ip address
        :type: str
        """
        self._ip = ip
        self._continent = None
        self._country = None
        self._geo_data = None
        self._raw_data = None

    def geolocate(self):
        """Call geoloaction api. Should be overriden."""
        raise NotImplementedError()

    def _parse(self):
        """Parse raw data and store them in object attributes."""
        raise NotImplementedError()

    def data(self):
        """Return the parsed data.

        :return: Parsed data.
        :rtype: dict
        """
        self._parse()

        return {
            'ip': self._ip,
            'continent': self._continent,
            'county': self._country,
            'geo': self._geo_data,
            'raw_data': self._raw_data
        }



"""IPGeolocationAPI.com service integration."""
import requests
from django_ip_geolocation.backends import GeolocationBackend


class IPGeolocationAPI(GeolocationBackend):
    """IPGeolocationAPI.com backend implementation."""

    def geolocate(self):
        """Call ipgeolocationapi api."""
        url = 'https://api.ipgeolocationapi.com/geolocate/{}'.format(self._ip)
        res = requests.get(url)
        if res.ok:
            self._raw_data = res.json()

    def _parse(self):
        """Parse raw data.

        raw_data example:
        {
            "continent": "Europe",
            "alpha2": "DE",
            "country_code": "49",
            "international_prefix": "00",
            "name": "Germany",
            "languages_spoken": [
                "de"
            ],
            "geo": {
                "latitude": 51.165691,
                "longitude": 10.451526,
            },
            "currency_code": "EUR"
        }
        """
        self._continent = self._raw_data.get('continent')
        self._country = {
            'code': self._raw_data.get('alpha2'),
            'name': self._raw_data.get('name'),
        }

        self._geo_data = self._raw_data.get('geo')
